﻿/// <reference path="../background.js" />

/** Customisation work for Battery Venture */
var gBatteryVentureCustomisation = (function () {
    var isCustomBarEnabled = false; 

    var blackListSites = [".facebook.", "www.google.", ".youtube.", ".wikipedia.", ".reddit.", ".yahoo.",
        ".amazon.", "twitter.", ".instagram.", ".bing.", ".ebay.", ".whatsapp.", ".msn.", ".spotify.",
        "1password.com", "451research.com", "aa.com", "adp.com", "airbnb.com", "alphasights.com",
        "amazonaws.com", "americanexpress.com", "thomsonreuters.com", "amtrak.com", "apple.com", "att.com",
        "azure.com", "ba.com", "bankofamerica.com", "battery.com", "betterment.com", "bloomberg.com",
        "bluecrossma.com", "bluejeans.com", "boingo.com", "bostonglobe.com", "bostonherald.com", "box.com", "businessinsider.com",
        "capitaliq.com", "capitalone.com", "carta.com", "chase.com", "cisco.com", "citi.com", "citibank.com",
        "clearbit.com", "cnbc.com", "cnn.com", "coinbase.com", "coinmarketcap.com", "comcast.com", "comcast.net", 
        "complysci.com", "concursolutions.com", "dashlane.com", "delta.com", "dell.com", "docusign.com", "docusign.net", "dropbox.com", "espn.com", "etrade.com",
        "eventbrite.com", "evernote.com", "fidelity.com", "fivethirtyeight.com", "forbes.com", "force.com",
        "fortune.com", "ft.com", "gartner.com", "github.com", "gmail.com", "gogoinflight.com", "google.com", "gotomeeting.com",
        "globalrelay.com", "groupme.com", "hertz.com", "hrblock.com", "hs-sites.com", "hubspot.com", "hyatt.com", "icloud.com", "ilevel.com",
        "ilevelsolutions.com", "instagram.com", "intralinks.com", "jetblue.com", "kayak.com", "lastpass.com", "marriott.com",
        "mergermarket.com", "merrillcorp.com", "microsoft.com", "microsoftonline.com", "netagesolutions.com", "networksolutions.com",
        "nationalcar.com", "netflix.com", "nike.com", "nytimes.com", "office.com", "office365.com", "okta.com", "optumbank.com", ".org", 
        "outlook.com", "paypal.com", "pitchbook.com", "pocket.com", "producthunt.com", "pureinsurance.com", "reddit.com", "rivals.com",
        "rcn.com", "salesforce.com", "schwab.com", "seamless.com", "sharepoint.com", "slack.com", "sourcescrub.com", "southwest.com",
        "spirit.com", "starwoodhotels.com", "statefarm.com", "stratechery.com", "stripe.com", "stubhub.com", "techcrunch.com",
        "ticketmaster.com", "twitter.com", "united.com", "unitedwifi.com", "usaa.com", "venmo.com", "venturebeat.com",
        "verizon.com", "verizonwireless.com", "walmart.com", "wayfair.com", "Wealthfront.com", "webex.com", "wellsfargo.com",
        "wsj.com", "ycombinator.com", "yesware.com", "zendesk.com", "zocdoc.com", "zoom.us", ".gov"];

    /**
     * Checks the layoutmanager to confirm the user is a user of battery ventures
     * @param {object} layoutManager - The layoutmanager object
     * @param {any} rightClickMenuOption - The menu option for Right Click Searcg which we'll disable
     */
    function Initialize(layoutManager, rightClickMenuOption) {
        if (layoutManager && layoutManager.d && layoutManager.d.CompanySettings) {
            isCustomBarEnabled = layoutManager.d.CompanySettings.CustomBarForAccounts === true;

            //Remove right click search if we're setting up a custom search
            if (isCustomBarEnabled === true) {
                if (rightClickMenuOption !== null) {
                    try {
                        chrome.contextMenus.removeAll();
                    } catch (error) {
                        console.error(error);
                    }
                }

                //Add some custom blacklist rules for popular websites that are used often to help prevent any annoyance
                var parsedBlacklistArray = [];                
                if (localStorage.SwitchOffBarWebsites) {
                    parsedBlacklistArray = JSON.parse(localStorage.SwitchOffBarWebsites);
                }

                if (Array.isArray(parsedBlacklistArray)) {
                    for (var i = 0; i < blackListSites.length; i++) {
                        if (parsedBlacklistArray.findIndex(site => site.toLowerCase() === blackListSites[i].toLowerCase()) === -1) {
                            parsedBlacklistArray.push(blackListSites[i]);
                        }
                    }

                    localStorage.SwitchOffBarWebsites = JSON.stringify(parsedBlacklistArray);
                }
            }
        }
    }

    /** The type of search we need to use depending on what site they're on */
    const SearchType = {
        /**
         * This uses the url of the page as the search term to search against an Accounts domain in Salesforce
         */
        URL: 1, 
        /**
         * This uses the company name on a linkedin profile to match to a Account record in Salesforce
         */
        LINKEDINCOMPANY: 2,
        /**
         * This searches for a contact, by name, in Salesforce returning their associated account
         */
        LINKEDINUSER : 3
    }

    /**
     * Using the search type triggered we'll return a type of search we need to use
     * @param {string} barSearchType -  The type of bar search that was triggered (Represented by the BarSearchTypes object)
     * @returns {int} - The type of search needed to be used
     */
    function GetSearchTypeFromBarSearchType(barSearchType) {
        switch (barSearchType) {
            case BarSearchTypes.CustomBarAccount.URL:
                return SearchType.URL;
            case BarSearchTypes.CustomBarAccount.LINKEDINUSER:
                return SearchType.LINKEDINUSER;
            case BarSearchTypes.CustomBarAccount.LINKEDINCOMPANY:
                return SearchType.LINKEDINCOMPANY;
        }
    }

    /**
     * Runs our custom bar search request sending the data to originating requester
     * @param {string} url - The url that will be used for the search
     * @param {object} sendResponse - Callback used to communicate back to originating sender
     * @param {string} searchTerm - The string to use as a search if not using the URL search
     * @param {string} barSearchType - The type of bar search that was triggered (Represented by the BarSearchTypes object)
     */
    function RunAccountBarSearch(url, sendResponse, searchTerm, barSearchType) {      
        const searchType = GetSearchTypeFromBarSearchType(barSearchType);
        let searchValue = url;
        if (searchType !== SearchType.URL) {
            searchValue = searchTerm;
        }

        var searchFields = "[" + GenerateEntityField('SearchType', "" + searchType, '', 'false');
        searchFields += ',' + GenerateEntityField('SessionKey', '' + localStorage.SessionKey, '', 'false');
        searchFields += ',' + GenerateEntityField('EBActionTaken', '' + localStorage.Action, '', 'false');
        searchFields += ',' + GenerateEntityField('SiteURL', '' + localStorage.LastPageUrl, '', 'false') + "]";

        var varData = GetDefaultRequestParameters().replace("}", "") +
            ", \"searchName\":" + JSON.stringify(searchValue) +
            ", \"entityToSearch\": \"customaccountsearch\"" +
            ", \"searchFields\": " + JSON.stringify(searchFields) + " }";

        MakeAjaxCallToSecure("POST", localStorage.CrmUrl + "/SearchV2", varData, "application/json; charset=utf-8", "json", sendResponse, sendResponse);
    }

    return {
        Initialize: Initialize,
        IsCustomBarEnabled: function() {
            return isCustomBarEnabled;
        },
        RunAccountBarSearch: RunAccountBarSearch
    };
})();