﻿/**
 * Closure to contain contents within the modal as well as the modal itself
 * @param {} hideModalFunctions - This is a list of functions that need to be called when HideModal is called
 * @returns {} 
 */
function EbstaModal(hideModalFunctions) {
    var gModalWindow = null,
        gConfirmationScreen = null,
        gConfirmationContent = null,
        gCancelButton = null,
        gConfirmButton = null,
        gOkButton = null;

    //Create the modal on initialising the closure
    gModalWindow = CreateElement("div", null, null, "ebsta-modal");
    gConfirmationScreen = CreateElement("div", null, null, "ebsta-confirmation-dialog");

    gConfirmationContent = CreateElement("section", null, null, "ebsta-confirmation-dialog-content");
    gHeaderSection = CreateElement("section", null, null, "ebsta-confirmation-dialog-content-header");
    gHeaderTitle = CreateElement("h2",null);
    gCloseDialogIcon = CreateElement("span", "sftitle-close");

    gContentBody = CreateElement("section", null, null, "ebsta-confirmation-dialog-content-body");
    gContentBodyText = CreateElement("span");

    var buttonsSection = CreateElement("section", null, null, "ebsta-confirmation-dialog-buttons");
    gCancelButton = CreateElement("button", "ebsta-new-btn ebsta-new-btn-secondary", "Cancel", "ebsta-confirmation-dialog-cancel-button", { type: "button" });
    gConfirmButton = CreateElement("button", "ebsta-new-btn", "Confirm", "ebsta-confirmation-dialog-confirm-button", { type: "button" });
    gOkButton = CreateElement("button", "ebsta-new-btn", "Ok", "ebsta-confirmation-dialog-ok-button", { type: "button" });

    buttonsSection.appendChild(gCancelButton);
    buttonsSection.appendChild(gConfirmButton);
    buttonsSection.appendChild(gOkButton);

    gHeaderSection.appendChild(gHeaderTitle);
    gHeaderSection.appendChild(gCloseDialogIcon);
    gConfirmationContent.appendChild(gHeaderSection);

    gContentBody.appendChild(gContentBodyText);
    gConfirmationContent.appendChild(gContentBody);

    gConfirmationScreen.appendChild(gConfirmationContent);
    gConfirmationScreen.appendChild(buttonsSection);
    gModalWindow.appendChild(gConfirmationScreen);
    document.body.appendChild(gModalWindow);

    HideElement(gConfirmationScreen);

    //Hides the text dialog section within the modal
    function hideDialog() {
        HideElement(gConfirmationScreen);
    }

    /** 
     * Essentially hides the modal and dialog, as well as resetting the state read to be used again. It 
     * also calls a list of functions if any were set on construction of the closure. 
     */
    function hideModal() {
        HideElement(gConfirmationScreen);
        HideElement(gModalWindow);

        setDefaultState();
        //Loop through all functions to call
        if (Array.isArray(hideModalFunctions)) {
            for (var i = 0; i < hideModalFunctions.length; i++) {
                hideModalFunctions[i]();
            }
        }
    }

    // Click event for the close dialog icon
    gCloseDialogIcon.addEventListener("click", () => {
        hideModal();
        hideDialog();
    });

    //Mousedown event on the modal overlay
    gModalWindow.addEventListener("mousedown", function (event) {
        if (event.target === gModalWindow) {
            hideModal();
        }
    });

    // Default Confirm button click event
    gConfirmButton.addEventListener("click", function () {
        hideModal();
        hideDialog();
    });

    //Cancel button click event
    gCancelButton.addEventListener("click", function () {
        hideModal();
        hideDialog();
    });

    //Check for key presses
    window.addEventListener("keyup", function (event) {
        //Escape key
        if (event.keyCode === 27 || event.key.toUpperCase() == "ESCAPE") {
            hideModal();
        }
    });

    // Default OK button click event
    gOkButton.addEventListener("click", function() {
        hideDialog();
        hideModal();
    });

    function setTitle(text = ""){
        gHeaderTitle.innerText = text;
    }

    function setBodyText(text = "") {
        gContentBodyText.innerText = text;
    }

    /** Shows just the OK button */
    function setOkButtonMode() {
        ShowElement(gOkButton);
        HideElement(gConfirmButton);
        HideElement(gCancelButton);
    }

    /** Shows Confirm and Cancel buttons only */
    function setConfirmButtonMode() {
        HideElement(gOkButton);
        ShowElement(gConfirmButton);
        ShowElement(gCancelButton);
    }

    /** Sets the OK button text */
    function setOkButtonText(text = "Ok") {
        gOkButton.innerText = text ? text : "Ok";
    }

    /** Sets the confirm button text */
    function setConfirmButtonText(text = "Confirm") {
        gConfirmButton.innerText = text ? text : "Confirm";
    }

    /** Sets the cancel button text */
    function setCancelButtonText(text = "Cancel") {
        gCancelButton.innerText = text ? text : "Cancel";
    }

    /** Setting up the click  event handler for the OK button */
    function handleOkButtonEvent(functionToCall) {
        gOkButton = setupNewClickEventHandleForButton(gOkButton, functionToCall);
    }

    /** Setting up the click event handler for the OK button */
    function handleConfirmButton(functionToCall) {
        gConfirmButton = setupNewClickEventHandleForButton(gConfirmButton, functionToCall);        
    }

    /** Setting up the click event handler for the Cancel button */
    function handleCancelButton(functionToCall) {
        gCancelButton = setupNewClickEventHandleForButton(gCancelButton, functionToCall);        
    }

    /**
     * Resets the given button by recereating the HTMLElement and then assigning a click listener to the new HTMLELEMENT
     * @param {HTMLElement} button - The element to clone and replace and assign new click listener to
     * @param {functionToCall} functionToCall - The function to call when new button is clicked
     * @returns {HTMLElement} - The new element thats been cloned with the new click event
     */
    function setupNewClickEventHandleForButton(button, functionToCall) {
        //Clone the node to remove all listeners easily
        const newElement = button.cloneNode(true);
        button.parentNode.replaceChild(newElement, button);
        button = newElement;

        button.addEventListener("click", function () {
            hideModal();
            hideDialog();
            if (typeof functionToCall === "function") {
                functionToCall();
            }
        });
        
        return button;
    }

    /** 
     * Resets everything back to original state and clears all text out to prevent unwanted/unexpected previous results showing
     */
    function setDefaultState() {
        setTitle();
        setBodyText();
        setCancelButtonText();
        setConfirmButtonText();
        setOkButtonText();
        setOkButtonMode();

        // Reset all button functionality to do nothing extra when clicked
        handleConfirmButton(() => { });
        handleCancelButton(() => { });
        handleOkButtonEvent(() => { });
    }

    setDefaultState();

    return {
        GetModal: function() {
            return gModalWindow;
        },
        ShowModal: function() {
            HideElement(gConfirmationScreen);
            ShowElement(gModalWindow);
        },
        HideModal: hideModal,
        SetText: function(text) {
            gContentBodyText.innerText = text;
            ShowElement(gConfirmationScreen);
        },
        HandleCancelButton: handleCancelButton,
        HideDialog: function() {
            hideDialog();
        },
        ShowDialog: function() {
            ShowElementBlock(gConfirmationScreen);
        },
        HandleConfirmButton: handleConfirmButton,
        HandleOkButton: handleOkButtonEvent,
        SetOkButtonText: setOkButtonText,
        SetConfirmButtonText: setConfirmButtonText,
        SetCancelButtonText: setCancelButtonText,
        SetOkButtonMode: setOkButtonMode,
        SetConfirmButtonMode : setConfirmButtonMode,
        SetTitle: setTitle
    }
}