﻿/**
* Ebsta own Xpath functionality
* @returns {} 
*/
var EbstaXpath = (function () {

    function htmlspecialchars_decode(string, quote_style) {
        // http://kevin.vanzonneveld.net
        // +   original by: Mirek Slugen
        // +   improved by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
        // +   bugfixed by: Mateusz "loonquawl" Zalega
        // +      input by: ReverseSyntax
        // +      input by: Slawomir Kaniecki
        // +      input by: Scott Cariss
        // +      input by: Francois
        // +   bugfixed by: Onno Marsman
        // +    revised by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
        // +   bugfixed by: Brett Zamir (http://brett-zamir.me)
        // +      input by: Ratheous
        // +      input by: Mailfaker (http://www.weedem.fr/)
        // +      reimplemented by: Brett Zamir (http://brett-zamir.me)
        // +    bugfixed by: Brett Zamir (http://brett-zamir.me)
        // *     example 1: htmlspecialchars_decode("<p>this -&gt; &quot;</p>", 'ENT_NOQUOTES');
        // *     returns 1: '<p>this -> &quot;</p>'
        // *     example 2: htmlspecialchars_decode("&amp;quot;");
        // *     returns 2: '&quot;'
        var optTemp = 0,
        i = 0,
        noquotes = false;
        if (typeof quote_style === 'undefined') {
            quote_style = 2;
        }
        string = string.toString().replace(/&lt;/g, '<').replace(/&gt;/g, '>');
        var OPTS = {
            'ENT_NOQUOTES': 0,
            'ENT_HTML_QUOTE_SINGLE': 1,
            'ENT_HTML_QUOTE_DOUBLE': 2,
            'ENT_COMPAT': 2,
            'ENT_QUOTES': 3,
            'ENT_IGNORE': 4
        };
        if (quote_style === 0) {
            noquotes = true;
        }
        if (typeof quote_style !== 'number') { // Allow for a single string or an array of string flags
            quote_style = [].concat(quote_style);
            for (i = 0; i < quote_style.length; i++) {
                // Resolve string input to bitwise e.g. 'PATHINFO_EXTENSION' becomes 4
                if (OPTS[quote_style[i]] === 0) {
                    noquotes = true;
                } else if (OPTS[quote_style[i]]) {
                    optTemp = optTemp | OPTS[quote_style[i]];
                }
            }
            quote_style = optTemp;
        }
        if (quote_style & OPTS.ENT_HTML_QUOTE_SINGLE) {
            string = string.replace(/&#0*39;/g, "'"); // PHP doesn't currently escape if more than one 0, but it should
            // string = string.replace(/&apos;|&#x0*27;/g, "'"); // This would also be useful here, but not a part of PHP
        }
        if (!noquotes) {
            string = string.replace(/&quot;/g, '"');
        }
        // Put this in last place to avoid escape being double-decoded
        string = string.replace(/&amp;/g, '&');

        return string;
    }

    /**
     * Gets the string for the element found
     * @param {} element 
     * @returns {} 
     */
    function ProcessStringElement(element) {
        if (element.stringValue) {
            return element.stringValue.trim();
        }
        return "";
    }

    /**
     * Takes a node and finds the inner content of it, returning a string
     * @param {} node 
     * @returns {} 
     */
    function GetTextContentFromNode(node) {
        if (node) {
            if (node.innerText) {
                return htmlspecialchars_decode(node.innerText).trim();
            }
            else if (node.value) {
                return htmlspecialchars_decode(node.value).trim();
            }
            else if (node.textContent) {
                return node.textContent;
            }
        }

        return "";
    }

    /**
     * Loops around all the XPathResult object and returns an array of them all
     * @param {} xPathResult 
     * @returns {} 
     */
    function GetNodeArrayFromResult(xPathResult) {
        var arrayOfNodesFound = new Array();

        var nextResult = xPathResult.iterateNext();
        while (nextResult) {
            arrayOfNodesFound.push(nextResult);
            nextResult = xPathResult.iterateNext();
        }

        return arrayOfNodesFound;
    }

    /**
     * Returns the contents of the first node found in the result
     * @param {} result 
     * @returns {} 
     */
    function ProcessNodeResult(result) {
        var arrayResult = GetNodeArrayFromResult(result);
        if (arrayResult.length > 0) {
            return GetTextContentFromNode(arrayResult[0]);
        }
        return "";
    }

    /**
     * Processes the element depending on what type was found and returns a string of its contents
     * @param {} element 
     * @returns {} 
     */
    function ProcessElement(element) {
        //https://developer.mozilla.org/en-US/docs/Web/API/Document/evaluate
        var typeFound = element.resultType;
        switch (typeFound) {
            case 2:
                return ProcessStringElement(element);
            case 4:
            case 5:
            case 6:
            case 7:
            case 8:
            case 9:
                return ProcessNodeResult(element);
            default:
                return "";
        }
    }

    /**
     * Runs an xpath evalutation returning back the XPathResult
     * @param {} xpath 
     * @param {} elementToSearch 
     * @param {} typeOfSearch 
     * @returns {} 
     */
    function RunXpath(xpath, elementToSearch, typeOfSearch) {
        try {
            if (xpath) {
                return document.evaluate(xpath, elementToSearch, null, typeOfSearch, null);
            }
        }
        catch (error) {
            console.log("XPath Err:", error);
        }
    }

    return {
        /**
         * Runs a path search returning back any type found and returns a string of the contains of the node found
         * @param {} xpath 
         * @returns {} 
         */
        GetStringByAnyType: function(xpath) {
            var xpathResult = RunXpath(xpath, document, XPathResult.ANY_TYPE);

            if (xpathResult) {
                return ProcessElement(xpathResult);
            }

            return "";
        },
        /**
         * Using an xpath expression, this will search and return back the first node found within the DOM
         * @param {} xpath 
         * @returns {} 
         */
        GetFirstNode: function (xpath) {
            var xpathResult = RunXpath(xpath, document, XPathResult.FIRST_ORDERED_NODE_TYPE);

            if (xpathResult && xpathResult.singleNodeValue) {
                return xpathResult.singleNodeValue;
            }

            return null;
        }
    }
})();