﻿/**
 * Helper module containing tools we'll use for anything in Gmail
 * @returns {} 
 */
var GmailTools = (function GmailTools() {
    var currentLoggedInUsername = null,
        isResizeListenerSet = false,
        isClickDecisionHandlerSet = false,
        gmailGlobals = null,
        isProxyInjected = false,
        isBarOnlyMode = false; // this is an option for gmail to run bardrops only

    /**
     * Contains functions we'll use for actions within a gmail compose window
     * @returns {} 
     */
    var compose = (function Compose() {

        /** Recipient types avaliable in the compose window */
        var recipientAddressTypes = {
            To: "to",
            Cc: "cc",
            Bcc: "bcc"
        }

        /**
         * takes in span element from gmail compose window and extracts firstname, lastname and email address 
         * @param {} element 
         * @returns {} 
         */
        function ExtractRecipientDetailsFromElement(element) {
            function CleanUpName(name) {

                var regexBrackets = /[()]/gi,
                    indicesBrackets = [],
                    resultBrackets = null;
                while ((resultBrackets = regexBrackets.exec(name))) {
                    indicesBrackets.push(resultBrackets.index);
                }

                if (indicesBrackets.length === 2) {
                    if (indicesBrackets[1] !== name.length) {
                        name = name.substring(0, indicesBrackets[0]) + name.substring(indicesBrackets[1] + 1);
                    } else {
                        name = name.substring(0, indicesBrackets[0]);
                    }
                }

                var regex = /[<>]/gi,
                    indicess = [],
                    result = null;
                while ((result = regex.exec(name))) {
                    indicess.push(result.index);
                }

                if (indicess.length === 2) {
                    if (indicess[1] !== name.length) {
                        name = name.substring(0, indicess[0]) + name.substring(indicess[1] + 1);
                    } else {
                        name = name.substring(0, indicess[0]);
                    }
                }

                if (name.includes(',')) {
                    var commaSplit = name.split(',');

                    if (commaSplit.length > 1) {
                        name = commaSplit[1].trim() + " " + commaSplit[0].trim();
                    }
                }

                name = name.replace(/\./g, '');

                return name.trim();
            }

            if (element.hasAttribute('email')) {
                var emailAddress = element.getAttribute('email'),
                    firstName = "",
                    lastName = "";

                var nameDiv = element.getElementsByClassName('vT');
                if (nameDiv && nameDiv.length > 0) {
                    var fullName = nameDiv[0].innerText;
                    if (fullName && fullName.length > 0) {
                        fullName = fullName.trim();
                        fullName = CleanUpName(fullName);
                        var names = fullName.split(' ');
                        if (names.length > 1) {
                            lastName = names[names.length - 1];
                            if (names.length > 2) {
                                // we might have a title
                                var n1 = names[0].toLowerCase();
                                if (n1 === 'ms' || n1 === 'mr' || n1 === 'mrs' || n1 === 'dr' || n1 === 'miss' || n1 === 'doctor') {
                                    firstName = names[1];
                                } else {
                                    firstName = names[0];
                                }
                            } else {
                                firstName = names[0];
                            }
                        }
                    }
                }

                return new Contact(emailAddress, firstName, lastName);
            }

            return null;
        }

        /**
        * Returns the element which contains the whole of the Gmail compose window
        * @param {} elementWithinCompose 
        * @returns {} 
        */
        function GetWholeComposeWindow(elementWithinCompose) {
            if (elementWithinCompose != null) {
                var classesToLookFor = [".Hd", ".aoP"];
                for (var i = 0; i < classesToLookFor.length; i++) {
                    var composeWindow = elementWithinCompose.closest(classesToLookFor[i]);
                    if (composeWindow) {
                        return composeWindow;
                    }
                }
            }
            return null;
        }

        /**
         * Takes an element within the dom of a compose window and searches within the dom to find that draftid element for the compose window
         * @param {} gmailComposeWindow 
         * @returns {} 
         */
        function FindDraftIdElement(elementWithinCompose) {
            var gmailComposeWindow = GetWholeComposeWindow(elementWithinCompose);
            if (gmailComposeWindow != null) {
                var draftElement = gmailComposeWindow.querySelector("input[name=draft]");
                if (draftElement != null) {
                    return draftElement;
                }
            }

            return null;
        }

        /**
        * Inserts the given Menu DOM into the bottom of the Compose Window above the Gmail Tool bar
        * @param { DOMElement } elementWithinGmailToolBar - Must be an element within the Gmail Toolbar
        * @param { DOMElement } menuDom - The menu you wish to append to the compose window
        * @returns {} 
        */
        function InsertMenu(elementWithinGmailToolBar, menuDom) {
            var gmailToolBar = elementWithinGmailToolBar.closest(".aDg");

            if (gmailToolBar) {
                //Find the container where the text toolbar within the compose window sits
                var findTheTextToolbarContainer = gmailToolBar.getElementsByClassName("aDl")[0];
                if (findTheTextToolbarContainer) {
                    findTheTextToolbarContainer.appendChild(menuDom);
                }
            }
        }

        /**
             * Runs regexs to find any tracking gifs within a piece of text
             * @param {} body 
             * @returns {} 
             */
        function RunRegexToFindTrackingGifWithinText(body) {
            //Check for it encoded first
            var id = /ebsta\.com\%2F([a-z0-9\-]*)\%2Febsta\.gif/g.exec(body);
            if (!id) {
                //Try unencoded
                id = /ebsta\.com\/([a-z0-9\-]*)\/ebsta\.gif/g.exec(body);
            }

            return id;
        }

        return {
            RecipientAddressTypes: recipientAddressTypes,
            RunRegexToFindTrackingGifWithinText: RunRegexToFindTrackingGifWithinText,
            FindDraftIdElement: FindDraftIdElement,
            GetWholeComposeWindow: GetWholeComposeWindow,
            InsertMenu: InsertMenu,
            /**
             * Takes an element from within a compose window and finds all contacts in that window
             * @param {} button 
             * @returns {} 
             */
            /**
             * Takes an element from within a compose window and finds all contacts in that window
             * @param {} composeElement - An element associted with the compose window in question
             * @param {} listOfRecipientAddressTypes -
             * @returns {} 
             */
            GetContactsFromComposeWindow: function (composeElement, listOfRecipientAddressTypes) {

                //Set default to find all types of recipients if nothing is specified
                if (listOfRecipientAddressTypes == null) {
                    listOfRecipientAddressTypes = [recipientAddressTypes.To, recipientAddressTypes.Cc, recipientAddressTypes.Bcc];
                }

                var contactArray = new Array();
                var outerTable = GetWholeComposeWindow(composeElement);
                if (outerTable) {
                    //Search for the container where all email addresses are held within the compose window
                    var recipientContainer = outerTable.getElementsByClassName("fX aiL")[0];
                    if (!recipientContainer) {
                        recipientContainer = outerTable.getElementsByClassName("fX XjCH")[0];
                    }
                    if (recipientContainer) {
                        var emailContainer = recipientContainer.querySelectorAll(".vR");
                        if (emailContainer && emailContainer.length > 0) {
                            for (var i = 0; i < emailContainer.length; i++) {

                                //Find what type of address this is e.g. To/Cc/Bc
                                var inputElement = emailContainer[i].getElementsByTagName("input")[0];
                                if (inputElement) {
                                    var typeName = inputElement.getAttribute("name");

                                    //Check this type has been specified for selection
                                    if (listOfRecipientAddressTypes.indexOf(typeName.toLowerCase()) != -1) {
                                        var contactSpan = emailContainer[i].getElementsByClassName('vN')[0];
                                        if (contactSpan) {
                                            var contactObj = ExtractRecipientDetailsFromElement(contactSpan);
                                            if (contactObj) {
                                                contactArray.push(contactObj);
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                return contactArray;
            },

            /**
             * Inserts an overlay over the element given matching the height and width of that element
             * @param {} domToAppendBefore 
             * @param {} messageText 
             * @returns {} 
             */
            InsertOverlay: function (domToAppendBefore, messageText) {
                if (domToAppendBefore != null) {
                    var loadingOverlayDiv = CreateElement("div", "ebLoadingOverlay", null, null);
                    var textAndLoaderDiv = CreateElement("div", "ebLoadingOverlayInfo", null, null);
                    textAndLoaderDiv.appendChild(CreateTextNode(messageText));
                    textAndLoaderDiv.appendChild(CreateBlueSpanSpinnerLoader());
                    loadingOverlayDiv.appendChild(textAndLoaderDiv);
                    if (domToAppendBefore.parentNode) {
                        domToAppendBefore.parentNode.insertBefore(loadingOverlayDiv, domToAppendBefore);
                    }
                    return loadingOverlayDiv;
                }
            },

            /**
             * Takes an element within the dom of a compose window and searches within the dom to find that draftid value for the compose window
             * @param {} gmailComposeWindow 
             * @returns {} 
             */
            GetDraftMessageIdValueFromCompose: function (elementWithinCompose) {
                var draftElement = FindDraftIdElement(elementWithinCompose);
                if (draftElement != null) {
                    var messageId = draftElement.getAttribute("value");
                    if (messageId != null && messageId !== "undefined") {
                        //New Gmail Ui 07/05/2018 - Format has changed to "#msg-a:r2410542770485228211"
                        if (messageId.indexOf("#msg-a:") === 0) {
                            messageId = messageId.replace("#msg-a:", "");
                        }
                        return messageId;
                    }
                }

                return null;
            },

            /**
             * Gets the compose windows trash icon from an element given that needs to be within that compose window
             * @param {} composeContainerDiv 
             * @returns {} 
             */
            GetTrashButtonFromComposeWindow: function (elementWithinCompose) {
                var composeContainerDiv = GetWholeComposeWindow(elementWithinCompose);
                if (composeContainerDiv != null) {
                    var trashButton = composeContainerDiv.getElementsByClassName("oh J-Z-I J-J5-Ji T-I-ax7");

                    //BackUp trash button finder in case we don't find it above
                    if (trashButton.length === 0) trashButton = composeContainerDiv.querySelector('div[data-tooltip="Discard draft"]');

                    return trashButton;
                }
                return null;
            },

            /**
             * Searchs for a compose window's text then inserts an element so it triggers a change on the DOM, 
             * which in turn fires triggers like updating the draft id
             * @param {} elementWithinCompose 
             * @returns {} 
             */
            TriggerChangeOnCompose: function (elementWithinCompose) {
                var gmailComposeWindow = GetWholeComposeWindow(elementWithinCompose);
                if (gmailComposeWindow) {
                    var composeWindowTextElements = gmailComposeWindow.getElementsByClassName("editable");
                    if (composeWindowTextElements.length > 0) {
                        var composeWindowElement = composeWindowTextElements[0];
                        if (composeWindowElement != null) {
                            composeWindowElement.appendChild(CreateElement("span", null, null, null));
                            SimulateKeyEventOnElement("keydown", composeWindowElement, "Q", "Q");
                        }
                    }
                }
            }
        }
    })();

    /**
     * Functionality around Gmail when in the view of looking at messages
     */
    var messageView = function () {

        /**
         * Finds all the message ids visible on the current page
         * @returns {} 
         */
        function FindMessageIds() {
            var messageArray = new Array();
            //This is a very general selector, as this is a new Gmail feature and the gmail classes may not be concrete as of yet
            var messageElements = document.querySelectorAll("[data-message-id]");
            for (var i = 0; i < messageElements.length; i++) {
                messageArray.push(messageElements[i].getAttribute("data-message-id"));
            }
            return messageArray;
        }

        /**
         * Finds all the legacy message ids visible on the current page (i.e. 164f7c2e9bf630ef)
         * @returns {} 
         */
        function FindAllLegacyIds() {
            var messageArray = new Array();
            //This is a very general selector, as this is a new Gmail feature and the gmail classes may not be concrete as of yet
            var messageElements = document.querySelectorAll("[data-legacy-message-id]");
            for (var i = 0; i < messageElements.length; i++) {
                messageArray.push(messageElements[i].getAttribute("data-legacy-message-id"));
            }
            return messageArray;
        }

        /**
         * Detects if this is message view
         * @returns {} 
         */
        function IsThisAMessageView() {
            //Detects if a user has landed on a email page, i.e. https://mail.google.com/mail/u/0/#inbox/144ea4abc8acf9ae
            //Lets look for an id within the link using regex so we know this an actual email page the user is on
            var documentSplitBySlash = document.URL.split("/");
            var regExp = /^(?=.*[a-z])(?=.*\d)[a-z\d]{16}/g; //Looks for a string that has at least 1 number and letter with 16 chraracters e.g. 144ea4abc8acf9ae
            for (var i = documentSplitBySlash.length - 1; i >= 0; i--) {
                var matches = documentSplitBySlash[i].match(regExp);
                if (matches != null) {
                    if (matches.length > 0) {
                        return true;
                    }
                }
            }

            //NEW GMAIL UI - 04/05/2018
            return FindMessageIds().length > 0;
        }

        /**
         * Takes a string a looks for the Google message ID within that string
         * @param {} stringToCheck 
         * @returns {} 
         */
        function FindMessageIdInString(stringToCheck) {
            var msgId = "";
            if (stringToCheck != null) {
                var messageId = stringToCheck.match(/\bm[A-Za-z0-9]{16}\b/g);
                //The first char will be m, this is not a part of the Id, so get rid of it.
                if (messageId != null) {
                    if (messageId[0] != null) {
                        if (messageId[0].charAt(0) == 'm') {
                            messageId = messageId[0].slice(1);
                            msgId = messageId;
                        }
                    }
                }
            }
            return msgId;
        }

        /**
         * Find the new format message id within the message view of an email i.e. (#msg-a:r-2171597239223774007)
         * @param {DOMElement} elementWithinMessageDom - This must be an element that sits within that message itself and be a child element of where the id sits
         * @returns {string} - The id in the new format  i.e (#msg-a:r-2171597239223774007)
         */
        function GetNewFormatMessageId(elementWithinMessageDom) {
            if (elementWithinMessageDom) {
                const element = elementWithinMessageDom.closest(".ads");
                if (element) {
                    return element.getAttribute("data-message-id");
                }
            }
            return null;
        }

        /**
         * Find the message id within the message view of an email
         * @param {} elementWithinMessageDom - This must be an element that sits within that message
         * @returns {} 
         */
        function GetMessageId(elementWithinMessageDom) {
            var msgId;

            $idElement = $(elementWithinMessageDom).find('.adP');

            //Find the messageId - the id is set as a class of the div with a class of .adP.
            var idElementClassString = "";

            //check we found the element.
            if ($idElement && $idElement.length > 0) {
                idElementClassString = $idElement.attr('class');
            } else {
                //if we did not find it, check again for trimless plugin class, as this changes the class name selctor.
                $idElement = $(elementWithinMessageDom).find('.trimless-adP');

                if ($idElement && $idElement.length > 0) {
                    idElementClassString = $idElement.attr('class');
                }
            }

            //The messageId element has serveral classes, the Id starts with m, get all alphanumerical chars after m and up to the first space
            msgId = FindMessageIdInString(idElementClassString);

            //Retry method implemented on 18th May as Gmail layout changed
            if (msgId.length === 0 || !ContainsNumber(msgId)) {
                idElementClassString = $(elementWithinMessageDom).find('.aXjCH').attr('class');
                msgId = FindMessageIdInString(idElementClassString);
            }

            //Extra check for New Gmail Ui - 04/05/2018
            if (!msgId) {
                msgId = GetLegacyIdFromElement(elementWithinMessageDom[0]);
            }

            return msgId;
        }

        /**
         * Returns the closest Gmail Legacy ID (164f7c2e9bf630ef) to the specified element
         * @param {any} element - An element within the DOM we should start our search at
         * @returns {string} - Id of the message
         */
        function GetLegacyIdFromElement(element) {
            if (element) {
                element = element.closest(".ads");
                if (element) {
                    return element.getAttribute("data-legacy-message-id");
                }
            }
            return null;
        }

        /**
         * Finds the message id associated with where the Ebsta buttons have been appended and then checks if this has already been added
         * @param {DOMElement} ebstaButtonDiv - Either the Quick Add or Complex Add button usedto add an email to salesforce
         * @param {Array} listOfGoogleIds - A list of google ids we need to use to find a match with the current email we're looking at
         * @returns {boolean} - True if we found a match
         */
        function HasEmailBeenAddedAlready(ebstaButtonDiv, listOfGoogleIds) {
            if (listOfGoogleIds) {
                if (listOfGoogleIds.length > 0) {
                    var foundMessage = false;
                    $(ebstaButtonDiv).closest('.gs').find('.adO,.aXjCH').each(function () {
                        var messageId = FindMessageIdsWithinClassOfDiv($(this));
                        if (messageId !== null) {
                            if (listOfGoogleIds.indexOf(String(messageId)) !== -1) {
                                foundMessage = true;
                                return false;
                            }
                        }
                    });

                    if (foundMessage === false) {
                        let messageId = GetLegacyIdFromElement(ebstaButtonDiv[0]);
                        if (listOfGoogleIds.indexOf(messageId) !== -1) {
                            foundMessage = true;
                        }
                    }
                }
            }

            return foundMessage;
        }

        /**
         * Takes a div and looks at the classes associated and finds the Gmail message id if any
         * e.g. class="ii gt m14f079ceaffae28c adP adO"
         * @param {DOMElement} divToSearch - The DomElement to search for an GmailId ID from
         * @returns {string} - Id if we found it, null if not
         */
        function FindMessageIdsWithinClassOfDiv(divToSearch) {
            var className = $(divToSearch).attr('class');
            if (className) {
                var classNameSplit = className.split(' ');
                for (var i = 0; i < classNameSplit.length; i++) {
                    var splitInstance = classNameSplit[i];
                    if (splitInstance.length > 16) {
                        if (splitInstance.substring(0, 1).toLowerCase() === "m") {
                            splitInstance = splitInstance.slice(1, splitInstance.length);
                            return splitInstance;
                        }
                    }
                }
            }
            return null;
        }

        /** 
         *  Finds as many ids on the page in the format like - 164f7c2e9bf630ef
         * @returns {Array} - List of Gmail Message Id's
         */
        function FindMessageIdsOnGmailPage() {

            var ids = new Array();

            //Find the id format is m14f079d2249b4019
            $('.adO,.aXjCH').each(function () {
                var emailId = FindMessageIdsWithinClassOfDiv($(this));
                if (emailId !== null) {
                    ids.push(emailId);
                }
            });

            var legacyIds = FindAllLegacyIds();
            if (legacyIds.length > 0) {
                for (let i = 0; i < legacyIds.length; i++) {
                    if (ids.indexOf(legacyIds[i]) === -1) {
                        ids.push(legacyIds[i]);
                    }
                }
            }

            return ids;
        }

        return {
            FindMessageIdsOnGmailPage: FindMessageIdsOnGmailPage,
            GetLegacyIdFromElement: GetLegacyIdFromElement,
            GetMessageId: GetMessageId,
            GetNewFormatMessageId: GetNewFormatMessageId,
            HasEmailBeenAddedAlready: HasEmailBeenAddedAlready,
            IsThisAMessageView: IsThisAMessageView,
            /**
             * Gets content of last email in a thread, in string form
             * @returns {} 
             */
            GetLastEmailInThread: function () {
                var foundNode = EbstaXpath.GetFirstNode("//div[contains(@class,'h7')][last()]//div[contains(@class,'aXjCH') " +
                    "and not(contains(@class,'undefined'))]");

                if (foundNode && foundNode.innerHTML) {
                    var tempElement = CreateElement("div");
                    tempElement.innerHTML = foundNode.innerHTML.replace(/<\/p>/g, "<\/p><br><br>");
                    return tempElement.innerText;
                }

                return "";
            }
        }
    }();

    /**
     * Returns the already set GLOBALS variable we retrieved from gmail
     * @returns {} 
     */
    function GetGmailGlobalsVariable() {
        return gmailGlobals ? gmailGlobals : null;
    }

    /**
     * Loops through an array to find an email
     * @param {} arrayOfValues 
     * @returns {} 
     */
    function LoopThroughValuesToFindEmail(arrayOfValues) {
        for (var i = 0; i < arrayOfValues.length; i++) {
            var globalValue = arrayOfValues[i];
            if (ValidateEmail(globalValue)) {
                return globalValue;
            }
        }
        return null;
    }

    /**
     * Looks for the GLOBALS to find a valid email, of which SHOULD be the username of the logged in user
     * @returns {} 
     */
    function GetUserNameFromGlobalsVariable() {
        var globals = GetGmailGlobalsVariable();
        if (globals) {
            //get the email from position 11 in the global variable array.
            if (globals.length > 10) {
                var userEmail = globals[10];

                //if the order has changed and this is not the email, loop through the glabal array to find it.
                if (ValidateEmail(userEmail)) {
                    currentLoggedInUsername = userEmail;
                    return true;
                }
            } else if (Array.isArray(globals)) {
                currentLoggedInUsername = LoopThroughValuesToFindEmail(globals);
            }
        }
    }

    /**
     * Initial setup for Gmail, needs to return a bool so GoogleTools knows if it has succeeded in finding it what it needs to get up an running
     * @returns {} 
     */
    function Initialize() {
        GetUserNameFromGlobalsVariable();
        if (!currentLoggedInUsername) return false;
        window["GLOBAL_EBSTA_MODAL"] = new EbstaModal(new Array());
        return true;
    }

    /**
     * Parses HTTP requests sent by Gmail
     * @param {any} responseText - The body of the response
     * @param {any} requestUrl - The url of the request
     * @returns {object} - Parsed properties of a gmail http response request
     */
    function HttpRequestParser(responseText, requestUrl) {
        var parsedResponse = null;

        // Try and parse the body of the response first to determine if its a JSON, if not
        // then we're not aware of what we're dealing with here so just return out
        try {
            parsedResponse = JSON.parse(responseText);
        } catch (error) {
            return null;
        };

        /**
         * Finds the thread if within a response from a save email request
         * @param {object} emailSection - The object containing details about the email (taken from a gmail request)
         * @returns {string} e.g. thread-a:r-8764345104955589333
         */
        function FindThreadId(emailSection) {
            try {
                return emailSection["1"];
            } catch (error) {
                return null;
            }
        }

        /**
         * Finds the messageid if within a response from a save email request
         * @param {object} emailSection - The object containing details about the email (taken from a gmail request)
         * @returns {string} e.g. msg-a:r-763951519084948523
         */
        function FindMessageId(emailSection) {
            try {
                return emailSection["3"][0]["1"];
            } catch (error) {
                return null;
            }
        }

        /**
         * Finds the messageid in base10 format if within a response from a save email request
         * @param {object} emailSection - The object containing details about the email (taken from a gmail request)
         * @returns {string} e.g. 1600128984672390197
         */
        function FindBase10MessageId(emailSection) {
            try {
                return emailSection["3"][0]["2"]["29"];
            } catch (error) {
                return null;
            }
        }

        /**
         * Finds the messageid in base16 format if within a response from a save email request
         * @param {object} emailSection - The object containing details about the email (taken from a gmail request)
         * @returns {string} e.g. 164cee1316e4834e
         */
        function FindBase16MessageId(emailSection) {
            try {
                return emailSection[3][0][2][35];
            } catch (error) {
                return null;
            }
        }

        /**
         * Finds the body of the email
         * @param {object} emailSection - The object containing details about the email (taken from a gmail request)
         * @returns {string} - Returns the html for the body of the email
         */
        function FindBody(emailSection) {
            try {
                var emailBodyArray = emailSection["3"][0]["2"]["6"]["2"];

                if (emailBodyArray !== null && emailBodyArray !== undefined) {
                    return ExtractEmailHtml(emailBodyArray);
                } else {
                    return "";
                }
            } catch (error) {
                return null;
            }
        }

        /**
         * Email content html is split into chunks - loops through each item and rebuilds the html as a whole.
         * @param {array} emailBodyArray - An Array which should contain the HTML body in one of the elements
         * @returns {string} - Returns the full html for the body of the email
         */
        function ExtractEmailHtml(emailBodyArray) {

            var html = [];

            if (emailBodyArray !== null && emailBodyArray !== undefined) {
                for (var i = 0; i < emailBodyArray.length; i++) {
                    var htmlItem = emailBodyArray[i];

                    var sectionHtml = htmlItem["3"]["2"];

                    if (sectionHtml !== null && sectionHtml !== undefined) {
                        html.push(sectionHtml);
                    }
                }
            }

            return html.join("");
        }

        var messageProperties = new Array();

        // Look for urls containing u/0/i/fd or u/1/i/fd
        // I think the numbers in middle are representing what the session id of the particular logged in username
        // so if you have 5 sessions active this number could go up to 5, but it's a guess
        if (/\/i\/fd/.test(requestUrl)) {

            //Requests now seem to return a batch of emails that have been sent rather than 1 request for each email so find
            //the core property where there responses lie and we'll process each one
            var emailSectionArray = parsedResponse[2];

            if (Array.isArray(emailSectionArray)) {
                for (let i = 0; i < emailSectionArray.length; i++) {
                    var emailSection = emailSectionArray[i];

                    messageProperties.push({
                        RequestUrl: requestUrl,
                        MessageIdBase10: FindBase10MessageId(emailSection),
                        MessageIdBase16: FindBase16MessageId(emailSection),
                        MessageId: FindMessageId(emailSection),
                        ThreadId: FindThreadId(emailSection),
                        Body: FindBody(emailSection)
                    });
                }
            }
        }

        return {
            MessageProperties: messageProperties
        };
    }

    /**
     * Sets up the proxy by injecting the script to intercept requests as well as setting up a message listener to handle callbacks from values that are found
     * @returns {} 
     */
    function InjectProxy() {
        if (!isProxyInjected) {
            window.addEventListener("message", function (event) {
                // We only accept messages from ourselves
                if (event.source != window)
                    return;

                if (event.data.type && (event.data.type == "FROM_PAGE")) {
                    var resp = event.data.text;

                    //Old regex
                    //var regexMatch = /\[\"a\"\,1\,\"[a-zA-Z\-\s\.]*\"\,\[\"([a-z0-9]*)\"\,/gi;

                    //This regex would be overall safest choice. We are matching by ["a", "1", "anything but linebreak", ["lowercase and digits" 
                    //- that is narrow enough to be safe. But allows us to get around dangers with unicode chars. Javascript does not support
                    //unicode safe regex(?). Alternative is to add safe unicode ref. for each dangerous char.
                    ///\[\"a\"\,1\,\"[A-Za-z\u00E0\u00E8\u00E9\u00EC\u00F2\u00F3\u00F9\-\s\.]*\"\,\[\"([a-z0-9]*)\"\,/gi;
                    var regexMatch = /\[\"a\"\,1\,\".*\"\,\[\"([a-z0-9]*)\"\,/gi;
                    var emailTracking = /ebsta\.com\%2F([a-z0-9\-]*)\%2Febsta\.gif/g;

                    //Regex to extrac the compose id from the body of the request returned
                    //e.g =&to=%22Dr.%20robwhite%20asdasd%22%20%3Crobwhite%40ebsta.com%3E&to=&=&cc=&=&bcc=&subjectbox=asdasasd&composeid=1492604496776&from=&subject=asdasasd&draft=undefined
                    var composeIdRegex = /composeid=([0-9]*)&/gi.exec(resp);

                    var match = regexMatch.exec(resp);

                    if (match != null && composeIdRegex != null) {
                        var messageId = match[1];

                        var urlIdMatch = emailTracking.exec(resp);
                        UpdateComplexAddArrayBeforeSaveOnSend(messageId, composeIdRegex[1]);
                        UpdateLogTrackingArrayBeforeTracking(messageId, urlIdMatch);
                    }
                }

                if (event.data.type && (event.data.type === "GMAILHTTPREQUEST")) {
                    var httpRequestParser = new HttpRequestParser(event.data.text, event.data.url);
                    if (httpRequestParser.MessageProperties && httpRequestParser.MessageProperties.length > 0) {
                        for (let i = 0; i < httpRequestParser.MessageProperties.length; i++) {

                            var requestResponse = httpRequestParser.MessageProperties[i];

                            UpdateComplexAddArrayBeforeSaveOnSend(requestResponse.MessageIdBase16, null, requestResponse.MessageId, requestResponse.MessageIdBase10);

                            UpdateLogTrackingArrayBeforeTracking(requestResponse.MessageIdBase16, compose.RunRegexToFindTrackingGifWithinText(requestResponse.Body),
                                requestResponse.MessageIdBase10, requestResponse.MessageId);
                        }
                    }
                }

                if (event.data.type && (event.data.type == "NEWEMAIL")) {
                    LogThis("NEW EMAIL Recieved");
                    if (LoggedIn) {
                        chrome.extension.sendMessage({ MessageType: "GetAccountActive", EmailAddress: GetGmailUsersEmailAddress() }, function (response) {
                            if (response != null) {
                                if (response.AccountActive + "" === "true") {
                                    runCheckInboxWithRetry();
                                }
                            }
                        });
                    }
                }

                if (event.data != null && event.data.type != null) {
                    switch (event.data.type) {
                        case "DRAFT_RESPONSE":
                            //If send later is enabled lets check for any change in in drafts being updated to a new Id
                            if (abRules != null && abRules.EnableSendLater + "" == "1") {
                                HandleUpdatedDraftResponse(event.data);
                            }
                            break;
                    }
                }
            }, false);

            var g = document.createElement('script');
            g.src = chrome.extension.getURL('js/gmailproxy.js');
            (document.head || document.documentElement).appendChild(g);
            isProxyInjected = true;
        }
    }

    /**
     * Is the Message Id one we see in the new Gmail UI
     * @param {} id 
     * @returns {} 
     */
    function IsMessageIdForNewUi(id) {
        return id.indexOf("#msg") === 0 || id.startsWith("r");
    }

    /**
     * Removes the text not actually apart of an id
     * @param {} id 
     * @returns {} 
     */
    function ConvertMessageIdIntoFriendlyId(id) {
        return id.replace("#msg-a:", "");
    }

    /**
     * Detects if current window being using is an external gmail compose screen
     * @returns {} 
     */
    function IsThisAExternalGmailComposeScreen() {
        return window.location.href.indexOf("view=btop") !== -1 || window.location.href.indexOf("view=cm") !== -1;
    }

    /**
     * Removes all ebsta buttons from the view to prevent users for access features when not logged in
     * @returns {} 
     */
    function RemoveAllEbstaButtons() {
        $('.googleButtonStyle, .googleButtonStyleLarge, .ebstaTD').each(function () {
            $(this).remove();
        });
    }

    /**
     * Checks if the first row of the gmail email table has the ebstaFirstRowChecked class set.
     * If it does not, then the first row has been re rendered since the last time we ran CheckInbox,
     * therefor we should run this again, to apply the ebsta contact / lead icons.
     */
    function HaveWeRunASearchForTheseEmails() {
        let gmailEmailRows = $("[email]:visible");
        if (gmailEmailRows.length > 0) {
            let emailTableFirstRow = $(gmailEmailRows[0]).closest("tbody > tr:first-child");
            if (emailTableFirstRow.length > 0) {
                return $(emailTableFirstRow[0]).hasClass("ebstaFirstRowChecked");
            }
        }
        //if we did not find the first row at all - then we assume the class is not set
        return false;
    }

    /**
     * Sets the ebstaFirstRowChecked class on the first row of the gmail email table.
     * We can then use this class in HaveWeRunASearchForTheseEmails() to see if we have checked
     * the first row of the email table for ebstas contact / lead icons. If the first row does not have this class
     * the table has been re rendered by gmail since the last time we ran CheckInbox(), so we should run it again
     * @param {any} firstEmailElement - the first element we have found in the gmail email table
     */
    function SetClassToStateEmailsHaveBeenChecked(firstEmailElement) {
        let emailTable = $(firstEmailElement).closest("tbody > tr:first-child");
        if (emailTable) {
            $(emailTable).addClass("ebstaFirstRowChecked");
        }
    }

    /**
     * Run CheckInbox() process with a retry framework that accounts for gmail
     * re rendering the email table. 
     * */
    var runCheckInboxWithRetry = function () {
        setTimeout(function () {
            //Runs a check to see if we have checked icons for the first row in the email table
            //if we have not, then we can just run check CheckInbox straight away - this means 
            //gmail has already re rendered the table and we are good to go
            if (!HaveWeRunASearchForTheseEmails()) {
                CheckInbox();
            } else {
                //if we have gotten to NEWEMAIL but our icons are still there, this
                //indicates that gmail has not yet re rendered the table, but they are about to
                //so we can go intry a retry to check the state of our icons over the next X seconds to ensure we are ok
                var retryMax = 6;
                var retryCount = 0;
                var iconRetryInternal = setInterval(function () {
                    if (retryCount < retryMax) {
                        //if during our retry we see that the first row does not have our check class
                        //this means it has been re rendered and we need to go check again
                        if (!HaveWeRunASearchForTheseEmails()) {
                            CheckInbox();
                            //if we have run a check during a retry then stop
                            clearInterval(iconRetryInternal);
                        }
                    } else {
                        clearInterval(iconRetryInternal);
                    }
                    retryCount++;
                }, 500);
            }
        }, 550);
    };


    /**
     * Apply the buttons for an email, complex add, quick add...
     * @param {any} emailAddress - Email Address
     * @param {any} id - Id
     * @param {any} type - Type
     * @param {any} server - Server
     * @param {any} multipleRecords - Multiple records
     * @param {any} url - Url
     * @param {any} name - Name
     * @param {any} accountInfo - Account info
     * @param {any} googleIds - Google ids
     */
    var applyStyleEmailStyleMsg = function (emailAddress, id, type, server, multipleRecords, url, name, accountInfo, googleIds) {
        var multiRecordSpan = '';
        var multiRecordFlag = '0';
        var openInSFClass = 'sfOpenLarge';
        var complexAddClass = 'sfComplexAdd';
        var multiRecordClass = '';

        var noAddEmailClass = '';

        if (type != "nocontact" && multipleRecords) {
            //multiRecordSpan = '<span class="multiRecordSpan" style="margin-left:4px!important;">+</span>';
            multiRecordFlag = '1';
            multiRecordClass = 'multiContactLarge';
        }

        if (url == document.URL) {

            var searchEmailAddress = emailAddress.replace("'", "\\'");

            var msgBody = $("*[email*='" + searchEmailAddress + "'");

            //Are we looking for the Sendee or the Sender - .gF =  Sender / .hb = sendee
            var childEmailElemToSearch = ".gF";
            if (url.toLowerCase().indexOf("sent") != -1) {
                childEmailElemToSearch = ".hb";
            }

            $(".Bu").find(childEmailElemToSearch).find(msgBody).each(function () {
                var foundUs = false;
                // if ($(this).parent().attr("class") == "iw") {
                var quickAddClass = 'sfQuickAdd';
                if (type == "nocontact") {
                    noAddEmailClass = 'sfnoemailadd';
                    quickAddClass = 'sfQuickAdded';
                    openInSFClass = '';
                    complexAddClass = 'sfComplexAdd';
                }

                try {
                    if (type == null) {
                        type = '';
                    }

                    var spn = CreateSpan("", "hk J-J5-Ji sfAddBt");
                    spn.setAttribute("style", "padding-right: 4px!important;");

                    //Check if email has been added already, if so change classes of icons to show the added icon
                    var idFoundInDB = false;
                    if (messageView.HasEmailBeenAddedAlready($(this), googleIds)) {
                        quickAddClass = "sfQuickAdded";
                        if (noAddEmailClass) complexAddClass = "sfComplexAdded";
                        idFoundInDB = true;
                    }

                    var spnQuickAdd = CreateSpan("", quickAddClass + ' ' + noAddEmailClass + ' googleButtonStyleLarge');
                    spnQuickAdd.setAttribute("sfEmailAddress", id);
                    spnQuickAdd.setAttribute("sfEmailAddressText", emailAddress);
                    spnQuickAdd.setAttribute("sfType", type);
                    spnQuickAdd.setAttribute("sfName", name);
                    spnQuickAdd.setAttribute("title", "Lightning-Add Email to Salesforce");



                    spn.appendChild(spnQuickAdd);

                    var spnComplexAdd = CreateSpan("", complexAddClass + ' googleButtonStyleLarge');
                    spnComplexAdd.setAttribute("sfEmailAddress", id);
                    spnComplexAdd.setAttribute("sfEmailAddressText", emailAddress);
                    spnComplexAdd.setAttribute("sfType", type);
                    spnComplexAdd.setAttribute("sfName", name);
                    spnComplexAdd.setAttribute("title", "Add Email to Salesforce");

                    //Look at the potential Account information passed through and attach onto the div so we can use it when a user opens the complex add screen.
                    if (accountInfo) {
                        if (accountInfo.Id) {
                            spnComplexAdd.setAttribute("whatId", accountInfo.Id);
                        }
                        if (accountInfo.Name) {
                            spnComplexAdd.setAttribute("whatName", accountInfo.Name);
                        }
                        spnComplexAdd.setAttribute("whatRecordType", "Account");
                    }

                    spn.appendChild(spnComplexAdd);

                    var spnOpen = CreateSpan("", 'sf' + type.toLowerCase() + 'Large ' + openInSFClass + ' googleButtonStyleLarge ' + multiRecordClass + ' ebsta-view-email-search-icon');
                    spnOpen.setAttribute("sfId", server + id);
                    spnOpen.setAttribute("instance", server);
                    spnOpen.setAttribute("sfEmailAddress", emailAddress);
                    spnOpen.setAttribute("mR", multiRecordFlag);
                    spnOpen.setAttribute("sfName", name);
                    spnOpen.setAttribute("title", "Search " + emailAddress + " in Salesforce");

                    spn.appendChild(spnOpen);

                    //Are the buttons there already???
                    var amountOfButtons = 0;

                    if ($(this).closest(".gE").find(".gH").find(".gK").find(".sfAddBt").length) {
                        amountOfButtons = $(this).closest(".gE").find(".gH").find(".gK").find(".sfAddBt").length;
                        //If the buttons are already there but we've found the ID lets make sure we change the icons]
                        if (amountOfButtons > 0) {
                            if (idFoundInDB) {
                                $(this).closest(".gE").find(".gH").find(".gK").find(".sfQuickAdd").removeClass('sfQuickAdd').addClass('sfQuickAdded');
                                //Only change complex add button if the lightning add button isn't being displayed
                                var complexAddButton = $(this).closest(".gE").find(".gH").find(".gK").find(".sfComplexAdd");
                                if ($(complexAddButton).prev().hasClass('noemailadd')) {
                                    $(complexAddButton).removeClass('sfComplexAdd').addClass('sfComplexAdded');
                                }
                            }
                        }
                    }

                    if (amountOfButtons == 0 || type != "nocontact") {
                        $(this).closest(".gE").find(".gH").each(function () {
                            //Increase the max height of this container helps the Ebsta Buttons not appear over the 
                            //recipients list when the screen width is small and / or there are many recipients for an email
                            $(this).css("max-height", "35px");
                            $(this).find(".gK").each(function () {
                                $(this).find(".sfAddBt").remove();
                                $(this).first().prepend(spn);
                                $(this).css("padding-right", "30px");
                                $(this).css("margin-top", "-5px");
                                return false;
                            });
                        });
                    }
                } catch (error) {

                }

            });

            //apply styling agains date and favourites section in email view.

            //date.
            $(".sfAddBt").next().next().css("position", "relative");
            $(".sfAddBt").next().next().css("display", "inline-block");
            $(".sfAddBt").next().next().css("top", "8px");

            //favourites icon.
            $(".sfAddBt").next().next().next().css("position", "relative");
            $(".sfAddBt").next().next().next().css("display", "inline-block");
            $(".sfAddBt").next().next().next().css("top", "-5px");
            $(".sfAddBt").next().next().next().css("right", "-10px");
        }
        //console.timeEnd('ApplyStyleEmailStyleMsg');

    }; 

    /**
    * Sets up the click decision handler for gmail.
    * This makes a decision of what ebsta function to trigger
    * based on the element that was clicked on the page.
    * @returns {void}
    */
    var setClickDecisionHandler = function () {

        if (!isClickDecisionHandlerSet) {
            $(document).ready(function () {

                

                AttachKeyDownListenerOnGmailCompose();

                $(document).on("mousedown", function (event) {

                    event.stopPropagation();

                    //  runSearch = false;
                    //  runSearchInboxMsg = false;
                    // var barDrop = true;
                    var timerOut = 1500;

                    //console.time('DecisionObjClick');
                    //Find preview pane if it exists
                    var dec = DecisionOnClick(event);
                    //LogThis('Decision is');
                    //LogThis(dec);

                    var msgId = 0;

                    if (dec.EmailTrack) {
                        if (event.toElement != null) {
                            InsertImgForGmailEmailTracking(event.toElement, GetSubjectLineInGmailComposeOrReplyWindow(event.toElement));
                            LoggingEmailSend();
                        }
                    }

                    if (dec.UndoGmailSend) {
                        RemoveGmailTrackingLog();
                    }

                    if (dec.RunSearchInbox) {

                        // Total Delay Timer for click events needing a longer wait period
                        // 750 is a default timer for all click events
                        var totalTimer = 750 + dec.DelayTimer;

                        var d = new Date();
                        //console.time('ClearTimeOut');

                        clearTimeout();
                        //console.time('ClearTimeOut');

                        setTimeout(function () {
                            CheckInbox();
                        }, totalTimer);

                    }

                    if (dec.RunMessageBox) {

                        //console.time('RunMsgBox');

                        //LogThis('AddGoogleSendButtons from decision object...');
                        //console.log("bar.js RunMessageBox");
                        clearTimeout();
                        setTimeout(function () {
                            CheckInboxMsg(dec.BarDrop);
                        }, timerOut);
                        //setTimeout(function() { AddGoogleSendButtons(); }, timerOut);
                        AttachKeyDownListenerOnGmailReply();
                        //console.timeEnd('RunMsgBox');

                    }

                    if (dec.GoogleButtonsCreate) {

                        //console.time('GoogleButtonsCreate');
                        AddButtonDelay();
                        AttachKeyDownListenerOnGmailReply();
                        //console.timeEnd('GoogleButtonsCreate');
                    }

                    if (dec.RunComposeSearch) {

                        //console.time('RunComposeSearch');
                        clearTimeout();
                        RunCheckComposeAndReplySendee(true, false);
                        //console.timeEnd('RunComposeSearch');
                    }

                    if (dec.RunReplyForwardSearch) {


                        // console.time('RunReplyForwardSearch');
                        clearTimeout();
                        setTimeout(function () { RunCheckComposeAndReplySendee(false, true); }, 200);
                        //console.timeEnd('RunReplyForwardSearch');
                    }

                    if (dec.CheckPlainText) {

                        //console.time('CheckPlainText');
                        DetectPlainTextModeChoice(event);
                        //console.timeEnd('CheckPlainText');
                    }
                });

                //LogThis('set window resize...');

                setWindowResizeListener();

                isClickDecisionHandlerSet = true;
            });
        }
    };

    /**
    * Sets a "resize" listener on the whole window
    * @returns {void}
    */
    var setWindowResizeListener = function () {
        if (!isResizeListenerSet) {
            window.addEventListener("resize", function (event) {
                SetupResizeWindowEventsForGmailCompose();
                ChangeOnboardingButtonOnResize();
                CampaignManagerResizing().ResizeCampaignManager();
            });
            isResizeListenerSet = true;
        }
    };

    return {
        ConvertMessageIdIntoFriendlyId: ConvertMessageIdIntoFriendlyId,
        SetVariableData: function (globals) {
            gmailGlobals = globals;
        },
        Initialize: Initialize,
        InjectProxy: InjectProxy,
        IsMessageIdForNewUi: IsMessageIdForNewUi,
        IsThisAExternalGmailComposeScreen: IsThisAExternalGmailComposeScreen,
        /**
         * Returns the Username for the currently logged in account
         * @returns {} 
         */
        GetUserName: function () {
            return currentLoggedInUsername;
        },
        MessageView: messageView,
        Compose: compose,
        /**
         * Sets a "resize" listener on the whole window
         * @returns {} 
         */
        SetWindowResizeListener: setWindowResizeListener,
        RemoveAllEbstaButtons: RemoveAllEbstaButtons,
        SetClassToStateEmailsHaveBeenChecked: SetClassToStateEmailsHaveBeenChecked,
        /**
         * Apply the buttons for an email, depending on the current isBarOnlyMode.
         * If set to bar only mode, will not add any of the buttons for email add, cadence, etc.
         * @param {any} obj - Result object of email search
         */
        ApplyStyleEmailStyleMsgConvertObject: function (obj) {
            if (!isBarOnlyMode) {
                applyStyleEmailStyleMsg(obj.LookEmailAddress, obj.Id, obj.Type, obj.Server, obj.FoundMultipleRecords, obj.Url, obj.Name, obj.Account, obj.GoogleIds);
            }
        },
        /**
         * Set gmail up to run in a bar only mode. 
         * Where the bar will be dropped and the search match icons displayed.
         * But none of the other functionality is enabled, e.g complex add, cadences etc etc.
         * */
        SetupGmailBarOnlyMode: function () {

            isBarOnlyMode = true;

            setClickDecisionHandler();

            runCheckInboxWithRetry();
        },
        /**
         * Sets up the click decision handler for gmail.
         * This makes a decision of what ebsta function to trigger 
         * based on the element that was clicked on the page.
         * @returns {}
         */
        SetClickDecisionHandler: setClickDecisionHandler
    };
})();