﻿/**
 * Module containing tools to handle google accounts.
 * */
var GoogleAccount = (function GoogleAccount() {
    return {
        /**
         * Creates a google account row, with account email and is active tickbox
         * for a google account.
         * @param {any} googleAccount - The google account
         * @returns {DOMElement} - Returns table row, or null if invalid input.
        */
        CreateGmailAccountRow: function (googleAccount) {

            // set null, if row not created then function returns null
            var rowDiv = null;

            if (googleAccount && googleAccount["GoogleAccountEmail"]) {

                var emailAddress = CheckStringLength(googleAccount["GoogleAccountEmail"], 50);

                if (emailAddress) {
                    //Create Row
                    rowDiv = document.createElement('tr');
                    rowDiv.setAttribute("class", "googleAccountTableRow");

                    var firstColumnDiv = document.createElement('td');
                    firstColumnDiv.setAttribute("class", "googleAccountTableRowColumnOne");

                    var emailDiv = document.createElement('div');
                    emailDiv.setAttribute("class", "googleAccountEmail");
                    emailDiv.innerText = emailAddress;

                    var secondColumnDiv = document.createElement('td');
                    secondColumnDiv.setAttribute("class", "googleAccountTableRowColumnTwo");

                    var checkBox = document.createElement('input');
                    checkBox.setAttribute("type", "checkbox");
                    checkBox.setAttribute("emailValue", emailAddress);

                    if (googleAccount["IsActive"] === 1) {
                        checkBox.setAttribute("checked", "checked");
                    }

                    if (googleAccount["DropBarWithNoAuth"] === true) {
                        checkBox.setAttribute("checked", "checked");
                        checkBox.setAttribute("data-isDropBarWithNoAuth", true);
                    }

                    //Email
                    firstColumnDiv.appendChild(emailDiv);

                    //Checkbox
                    secondColumnDiv.appendChild(checkBox);

                    rowDiv.appendChild(firstColumnDiv);
                    rowDiv.appendChild(secondColumnDiv);
                }
            }

            return rowDiv;
        },
        /**
         * Checks in local storage if a specified gmail account has been saved as being active by the user.
         * Or if it has been set with the DropBarWithNoAuth
         * @param {any} gmailAccountAddress - Email address of account to check
         * @returns {object} - { AccountActive: {bool}, DropBarWithNoAuth: {bool} }
         */
        CheckGmailAccountEnabled: function (gmailAccountAddress) {
            var accountActive = false;
            var dropBarWithNoAuth = false;

            if (gmailAccountAddress) {
                if (localStorage.GmailAccounts != null && typeof localStorage.GmailAccounts != "undefined") {
                    var gmailAccounts = JSON.parse(localStorage.GmailAccounts);
                    if (gmailAccounts.d != null) {
                        for (var i = 0; i < gmailAccounts.d.length; ++i) {
                            if (gmailAccounts.d[i].GoogleAccountEmail == gmailAccountAddress) {
                                //Set the local storage active flag to that of the current accounts active value
                                if (gmailAccounts.d[i].IsActive == '1') {
                                    accountActive = true;
                                }

                                if (gmailAccounts.d[i].DropBarWithNoAuth === true) {
                                    dropBarWithNoAuth = true;
                                }
                            }
                        }
                    }
                }
            }

            var returnObject = { AccountActive: accountActive, DropBarWithNoAuth: dropBarWithNoAuth };

            return returnObject;
        },
        /**
         * Generates a save item object for a google account, based on the checkbox input for that account.
         * @param {any} checkBox - The checkbox for a google account
         * @return {object} - { googleAccountEmail: string,  isActive: int, dropBarWithNoAuth: bool (optional) }
         */
        GenerateSaveObjectForGmailAccountCheckBox: function (checkBox) {

            var saveObject = null;

            if (checkBox) {
                var isActive = 0;
                if ($(checkBox).prop('checked')) {
                    isActive = 1;
                }

                var emailAddress = $(checkBox).attr('emailValue');

                if (emailAddress) {
                    saveObject = {
                        googleAccountEmail: emailAddress,
                        isActive: isActive
                    };

                    // If this check box has the isDropBarWithNoAuth attribute. The checkbox will be checked.
                    // But without the account being active, only having the isDropBarWithNoAuth option enabled.
                    // So we need to handle sligthly differently.
                    var isDropBarWithNoAuth = $(checkBox).attr('data-isDropBarWithNoAuth');
                    if (isDropBarWithNoAuth === "true") {
                        // If the checkbox is still checked - this means the user has made no change 
                        // So keep the actual account disabled, but dropBarWithNoAuth true
                        if (saveObject.isActive) {
                            saveObject.isActive = 0;
                            saveObject.dropBarWithNoAuth = true;
                        }
                        // If the checkbox has the attribute, it was checked only because isDropBarWithNoAuth was enabled
                        // But it has now been unchecked, this means the user wants to switch of this option also, so set to false.
                        else {
                            $(checkBox).removeAttr('data-isDropBarWithNoAuth');
                            saveObject.dropBarWithNoAuth = false;
                        }
                    }
                }
            }

            return saveObject;
        },
        /**
         * Updates the local storage for google account
         * @param {any} gmailAccount - Google account email address
         * @param {any} isActive - Flag for is active
         * @param {any} dropBarWithNoAuth - (optional) Flag for account drop bar with no auth
         * @return {void} - void
         */
        UpdateLocalStorage: function (gmailAccount, isActive, dropBarWithNoAuth) {
            var updateMade = false;

            if (gmailAccount != null && typeof gmailAccount != "undefined") {
                if (localStorage.GmailAccounts != null && typeof localStorage.GmailAccounts != "undefined") {
                    var gmailAccounts = JSON.parse(localStorage.GmailAccounts);
                    if (gmailAccounts.d != null) {
                        for (var i = 0; i < gmailAccounts.d.length; ++i) {
                            if (gmailAccounts.d[i].GoogleAccountEmail == gmailAccount) {

                                updateMade = true;
                                //Set the local storage active flag to that of the current accounts active value
                                gmailAccounts.d[i].IsActive = isActive;

                                if (dropBarWithNoAuth !== null && typeof dropBarWithNoAuth !== "undefined") {
                                    gmailAccounts.d[i].DropBarWithNoAuth = dropBarWithNoAuth;
                                }
                            }
                        }
                    }

                    if (updateMade) {
                        localStorage.GmailAccounts = JSON.stringify(gmailAccounts);
                    }
                    else {
                        gmailAccounts.d.push({ "__type": "GoogleAccount:#Culuru.Elemento.Common", "DateCreated": "/Date(1438105317780+0000)/", "GoogleAccountEmail": gmailAccount, "GoogleAccountId": "", "GoogleDontAuthNow": 0, "IsActive": isActive, "LastUpdated": "", "WebUserId": "" });
                        localStorage.GmailAccounts = JSON.stringify(gmailAccounts);
                    }
                }
            }
        }
    };
})();